@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import asyncio
 import contextlib
 import functools
@@ -7,20 +9,10 @@ import mimetypes
 import os
 import random
 import secrets
+import socketserver
 import sys
 import tempfile
-from typing import (
-    Any,
-    Awaitable,
-    Callable,
-    Dict,
-    List,
-    Optional,
-    Tuple,
-    TypeVar,
-    Union,
-    cast,
-)
+from typing import Any, Awaitable, Callable, Optional, TypeVar, cast
 
 if sys.version_info >= (3, 10):
     from typing import TypeGuard
@@ -32,6 +24,7 @@ if sys.version_info >= (3, 8):
 else:
     CancelledError = asyncio.futures.CancelledError
 
+
 # ==============================================================================
 # Misc utility functions
 # ==============================================================================
@@ -44,7 +37,7 @@ def rand_hex(bytes: int) -> str:
     return format_str.format(secrets.randbits(bytes * 8))
 
 
-def drop_none(x: Dict[str, Any]) -> Dict[str, object]:
+def drop_none(x: dict[str, Any]) -> dict[str, object]:
     return {k: v for k, v in x.items() if v is not None}
 
 
@@ -53,10 +46,10 @@ def drop_none(x: Dict[str, Any]) -> Dict[str, object]:
 # won't work for converting "top-level" lists to tuples
 def lists_to_tuples(x: object) -> object:
     if isinstance(x, dict):
-        x = cast(Dict[str, object], x)
+        x = cast("dict[str, object]", x)
         return {k: lists_to_tuples(v) for k, v in x.items()}
     elif isinstance(x, list):
-        x = cast(List[object], x)
+        x = cast("list[object]", x)
         return tuple(lists_to_tuples(y) for y in x)
     else:
         # TODO: are there other mutable iterators that we want to make read only?
@@ -64,7 +57,7 @@ def lists_to_tuples(x: object) -> object:
 
 
 def guess_mime_type(
-    url: Union[str, "os.PathLike[str]"],
+    url: "str | os.PathLike[str]",
     default: str = "application/octet-stream",
     strict: bool = True,
 ) -> str:
@@ -81,6 +74,54 @@ def guess_mime_type(
     return mimetypes.guess_type(url, strict)[0] or default
 
 
+def random_port(
+    min: int = 1024, max: int = 49151, host: str = "127.0.0.1", n: int = 20
+) -> int:
+    """Find an open TCP port
+
+    Finds a random available TCP port for listening on, within a specified range
+    of ports. The default range of ports to check is 1024 to 49151, which is the
+    set of TCP User Ports. This function automatically excludes some ports which
+    are considered unsafe by web browsers.
+
+    Parameters
+    ----------
+    min
+        Minimum port number.
+    max
+        Maximum port number, inclusive.
+    host
+        Before returning a port number, ensure that we can successfully bind it on this
+        host.
+    n
+        Number of times to attempt before giving up.
+    """
+
+    # fmt: off
+    # From https://github.com/rstudio/httpuv/blob/main/R/random_port.R
+    unsafe_ports = [1, 7, 9, 11, 13, 15, 17, 19, 20, 21, 22, 23, 25, 37, 42, 43, 53, 77, 79, 87, 95, 101, 102, 103, 104, 109, 110, 111, 113, 115, 117, 119, 123, 135, 139, 143, 179, 389, 427, 465, 512, 513, 514, 515, 526, 530, 531, 532, 540, 548, 556, 563, 587, 601, 636, 993, 995, 2049, 3659, 4045, 6000, 6665, 6666, 6667, 6668, 6669, 6697]
+    # fmt: on
+    unusable = set([x for x in unsafe_ports if x >= min and x <= max])
+    while n > 0:
+        if (max - min + 1) <= len(unusable):
+            break
+        port = round(random.random() * (max - min) + min)
+        if port in unusable:
+            continue
+        try:
+            # See if we can successfully bind
+            with socketserver.TCPServer(
+                (host, port), socketserver.BaseRequestHandler, bind_and_activate=False
+            ) as s:
+                s.server_bind()
+                return port
+        except Exception:
+            n -= 1
+            continue
+
+    raise RuntimeError("Failed to find a usable random port")
+
+
 # ==============================================================================
 # Private random stream
 # ==============================================================================
@@ -115,7 +156,7 @@ T = TypeVar("T")
 
 
 def wrap_async(
-    fn: Union[Callable[[], Awaitable[T]], Callable[[], T]]
+    fn: Callable[[], Awaitable[T]] | Callable[[], T]
 ) -> Callable[[], Awaitable[T]]:
     """
     Given a synchronous function that returns T, return an async function that wraps the
@@ -135,7 +176,7 @@ def wrap_async(
 
 
 def is_async_callable(
-    obj: Union[Callable[..., T], Callable[..., Awaitable[T]]]
+    obj: Callable[..., T] | Callable[..., Awaitable[T]]
 ) -> TypeGuard[Callable[..., Awaitable[T]]]:
     """
     Returns True if `obj` is an `async def` function, or if it's an object with a
@@ -163,8 +204,7 @@ def run_coro_sync(coro: Awaitable[T]) -> T:
     value. If it does not complete, then it will throw a `RuntimeError`.
 
     What it means to be "in fact synchronous": the coroutine must not yield
-    control to the event loop. A coroutine may have an `await` expression in it,
-    and that may call another function that has an `await`, but the chain will
+    control to the event loop. A coroutine may have an `await` expression in it, and that may call another function that has an `await`, but the chain will
     only yield control if a `yield` statement bubbles through `await`s all the
     way up. For example, `await asyncio.sleep(0)` will have a `yield` which
     bubbles up to the next level. Note that a `yield` in a generator used the
@@ -261,7 +301,7 @@ def run_coro_hybrid(coro: Awaitable[T]) -> "asyncio.Future[T]":
 # ==============================================================================
 class Callbacks:
     def __init__(self) -> None:
-        self._callbacks: dict[int, Tuple[Callable[[], None], bool]] = {}
+        self._callbacks: dict[int, tuple[Callable[[], None], bool]] = {}
         self._id: int = 0
 
     def register(
@@ -296,7 +336,7 @@ class Callbacks:
 
 class AsyncCallbacks:
     def __init__(self) -> None:
-        self._callbacks: dict[int, Tuple[Callable[[], Awaitable[None]], bool]] = {}
+        self._callbacks: dict[int, tuple[Callable[[], Awaitable[None]], bool]] = {}
         self._id: int = 0
 
     def register(
@@ -333,6 +373,7 @@ class AsyncCallbacks:
 # System-related functions
 # ==============================================================================
 
+
 # Return directory that a package lives in.
 def package_dir(package: str) -> str:
     with tempfile.TemporaryDirectory():
