@@ -1,6 +1,7 @@
-from shiny import *
-import numpy as np
 import matplotlib.pyplot as plt
+import numpy as np
+
+from shiny import *
 
 app_ui = ui.page_fluid(
     ui.input_slider("n", "Number of observations", min=0, max=1000, value=500),
@@ -14,7 +15,7 @@ def server(input: Inputs, output: Outputs, session: Session):
     @render.plot(alt="A histogram")
     # reactive.event() to invalidate the plot when the button is pressed but not when
     # the slider is changed
-    @reactive.event(lambda: input.go, ignore_none=False)
+    @reactive.event(input.go, ignore_none=False)
     def plot():
         np.random.seed(19680801)
         x = 100 + 15 * np.random.randn(input.n())
