@@ -6,6 +6,10 @@ app_ui = ui.page_fluid(
         ui.column(6, ui.output_text("text")),
         ui.column(6, ui.output_text_verbatim("verb", placeholder=True)),
     ),
+    ui.row(
+        ui.column(6),
+        ui.column(6, ui.output_text_verbatim("verb_no_placeholder", placeholder=False)),
+    ),
 )
 
 
@@ -20,5 +24,10 @@ def server(input: Inputs, output: Outputs, session: Session):
     def verb():
         return input.txt()
 
+    @output
+    @render.text
+    def verb_no_placeholder():
+        return input.txt()
+
 
 app = App(app_ui, server)
