@@ -1,3 +1,7 @@
+# Needed for types imported only during TYPE_CHECKING with Python 3.7 - 3.9
+# See https://www.python.org/dev/peps/pep-0655/#usage-in-python-3-11
+from __future__ import annotations
+
 __all__ = (
     "RenderFunction",
     "RenderFunctionAsync",
@@ -30,7 +34,6 @@ from typing import (
     Generic,
     Optional,
     TypeVar,
-    Union,
     overload,
 )
 
@@ -39,28 +42,21 @@ if sys.version_info >= (3, 8):
 else:
     from typing_extensions import Protocol, runtime_checkable
 
-from htmltools import TagChildArg
-
-# These aren't  used directly in this file, but they seems necessary for Sphinx to work
+# These aren't used directly in this file, but they seem necessary for Sphinx to work
 # cleanly.
-from htmltools import (  # pyright: ignore[reportUnusedImport] # noqa: F401
-    Tagifiable,  # pyright: ignore[reportUnusedImport]
-    Tag,  # pyright: ignore[reportUnusedImport]
-    TagList,  # pyright: ignore[reportUnusedImport]
-)
+from htmltools import Tag  # pyright: ignore[reportUnusedImport] # noqa: F401
+from htmltools import Tagifiable  # pyright: ignore[reportUnusedImport] # noqa: F401
+from htmltools import TagList  # pyright: ignore[reportUnusedImport] # noqa: F401
+from htmltools import TagChild
 
 if TYPE_CHECKING:
     from ..session import Session
     from ..session._utils import RenderedDeps
 
-from .._namespaces import ResolvedId
 from .. import _utils
+from .._namespaces import ResolvedId
 from ..types import ImgData
-from ._try_render_plot import (
-    try_render_matplotlib,
-    try_render_pil,
-    try_render_plotnine,
-)
+from ._try_render_plot import try_render_matplotlib, try_render_pil, try_render_plotnine
 
 # Input type for the user-spplied function that is passed to a render.xx
 IT = TypeVar("IT")
@@ -72,6 +68,7 @@ OT = TypeVar("OT")
 # RenderFunction/RenderFunctionAsync base class
 # ======================================================================================
 
+
 # A RenderFunction object is given a user-provided function which returns an IT. When
 # the .__call___ method is invoked, it calls the user-provided function (which returns
 # an IT), then converts the IT to an OT. Note that in many cases but not all, IT and OT
@@ -84,7 +81,7 @@ class RenderFunction(Generic[IT, OT]):
     def __call__(self) -> OT:
         raise NotImplementedError
 
-    def set_metadata(self, session: "Session", name: str) -> None:
+    def set_metadata(self, session: Session, name: str) -> None:
         """When RenderFunctions are assigned to Output object slots, this method
         is used to pass along session and name information.
         """
@@ -96,18 +93,18 @@ class RenderFunction(Generic[IT, OT]):
 # method is marked here as async; you can't have a single class where one method could
 # be either sync or async.
 class RenderFunctionAsync(RenderFunction[IT, OT]):
-    async def __call__(self) -> OT:  # type: ignore
+    async def __call__(self) -> OT:  # pyright: ignore[reportIncompatibleMethodOverride]
         raise NotImplementedError
 
 
 # ======================================================================================
 # RenderText
 # ======================================================================================
-RenderTextFunc = Callable[[], Union[str, None]]
-RenderTextFuncAsync = Callable[[], Awaitable[Union[str, None]]]
+RenderTextFunc = Callable[[], "str | None"]
+RenderTextFuncAsync = Callable[[], Awaitable["str | None"]]
 
 
-class RenderText(RenderFunction[Union[str, None], Union[str, None]]):
+class RenderText(RenderFunction["str | None", "str | None"]):
     def __init__(self, fn: RenderTextFunc) -> None:
         super().__init__(fn)
         # The Render*Async subclass will pass in an async function, but it tells the
@@ -115,43 +112,41 @@ class RenderText(RenderFunction[Union[str, None], Union[str, None]]):
         # passed an async function, it will not change it.
         self._fn: RenderTextFuncAsync = _utils.wrap_async(fn)
 
-    def __call__(self) -> Union[str, None]:
+    def __call__(self) -> str | None:
         return _utils.run_coro_sync(self._run())
 
-    async def _run(self) -> Union[str, None]:
+    async def _run(self) -> str | None:
         res = await self._fn()
         if res is None:
             return None
         return str(res)
 
 
-class RenderTextAsync(
-    RenderText, RenderFunctionAsync[Union[str, None], Union[str, None]]
-):
+class RenderTextAsync(RenderText, RenderFunctionAsync["str | None", "str | None"]):
     def __init__(self, fn: RenderTextFuncAsync) -> None:
         if not _utils.is_async_callable(fn):
             raise TypeError(self.__class__.__name__ + " requires an async function")
         super().__init__(typing.cast(RenderTextFunc, fn))
 
-    async def __call__(self) -> Union[str, None]:  # type: ignore
+    async def __call__(  # pyright: ignore[reportIncompatibleMethodOverride]
+        self,
+    ) -> str | None:
         return await self._run()
 
 
 @overload
-def text(fn: Union[RenderTextFunc, RenderTextFuncAsync]) -> RenderText:
+def text(fn: RenderTextFunc | RenderTextFuncAsync) -> RenderText:
     ...
 
 
 @overload
-def text() -> Callable[[Union[RenderTextFunc, RenderTextFuncAsync]], RenderText]:
+def text() -> Callable[[RenderTextFunc | RenderTextFuncAsync], RenderText]:
     ...
 
 
 def text(
-    fn: Optional[Union[RenderTextFunc, RenderTextFuncAsync]] = None
-) -> Union[
-    RenderText, Callable[[Union[RenderTextFunc, RenderTextFuncAsync]], RenderText]
-]:
+    fn: Optional[RenderTextFunc | RenderTextFuncAsync] = None,
+) -> RenderText | Callable[[RenderTextFunc | RenderTextFuncAsync], RenderText]:
     """
     Reactively render text.
 
@@ -171,7 +166,7 @@ def text(
     ~shiny.ui.output_text
     """
 
-    def wrapper(fn: Union[RenderTextFunc, RenderTextFuncAsync]) -> RenderText:
+    def wrapper(fn: RenderTextFunc | RenderTextFuncAsync) -> RenderText:
         if _utils.is_async_callable(fn):
             return RenderTextAsync(fn)
         else:
@@ -195,7 +190,7 @@ RenderPlotFunc = Callable[[], object]
 RenderPlotFuncAsync = Callable[[], Awaitable[object]]
 
 
-class RenderPlot(RenderFunction[object, Union[ImgData, None]]):
+class RenderPlot(RenderFunction[object, "ImgData | None"]):
     _ppi: float = 96
     _is_userfn_async = False
 
@@ -210,11 +205,10 @@ class RenderPlot(RenderFunction[object, Union[ImgData, None]]):
         # passed an async function, it will not change it.
         self._fn: RenderPlotFuncAsync = _utils.wrap_async(fn)
 
-    def __call__(self) -> Union[ImgData, None]:
+    def __call__(self) -> ImgData | None:
         return _utils.run_coro_sync(self._run())
 
-    async def _run(self) -> Union[ImgData, None]:
-
+    async def _run(self) -> ImgData | None:
         inputs = self._session.root_scope().input
 
         # Reactively read some information about the plot.
@@ -245,7 +239,7 @@ class RenderPlot(RenderFunction[object, Union[ImgData, None]]):
         # If a try_render function returns a tuple that starts with False, then the next
         # try_render function should be tried. If none succeed, an error is raised.
         ok: bool
-        result: Union[ImgData, None]
+        result: ImgData | None
 
         if "plotnine" in sys.modules:
             ok, result = try_render_plotnine(
@@ -287,7 +281,7 @@ class RenderPlot(RenderFunction[object, Union[ImgData, None]]):
         )
 
 
-class RenderPlotAsync(RenderPlot, RenderFunctionAsync[object, Union[ImgData, None]]):
+class RenderPlotAsync(RenderPlot, RenderFunctionAsync[object, "ImgData | None"]):
     _is_userfn_async = True
 
     def __init__(
@@ -297,12 +291,14 @@ class RenderPlotAsync(RenderPlot, RenderFunctionAsync[object, Union[ImgData, Non
             raise TypeError(self.__class__.__name__ + " requires an async function")
         super().__init__(typing.cast(RenderPlotFunc, fn), alt=alt, **kwargs)
 
-    async def __call__(self) -> Union[ImgData, None]:  # type: ignore
+    async def __call__(  # pyright: ignore[reportIncompatibleMethodOverride]
+        self,
+    ) -> ImgData | None:
         return await self._run()
 
 
 @overload
-def plot(fn: Union[RenderPlotFunc, RenderPlotFuncAsync]) -> RenderPlot:
+def plot(fn: RenderPlotFunc | RenderPlotFuncAsync) -> RenderPlot:
     ...
 
 
@@ -311,19 +307,17 @@ def plot(
     *,
     alt: Optional[str] = None,
     **kwargs: Any,
-) -> Callable[[Union[RenderPlotFunc, RenderPlotFuncAsync]], RenderPlot]:
+) -> Callable[[RenderPlotFunc | RenderPlotFuncAsync], RenderPlot]:
     ...
 
 
 # TODO: Use more specific types for render.plot
 def plot(
-    fn: Optional[Union[RenderPlotFunc, RenderPlotFuncAsync]] = None,
+    fn: Optional[RenderPlotFunc | RenderPlotFuncAsync] = None,
     *,
     alt: Optional[str] = None,
     **kwargs: Any,
-) -> Union[
-    RenderPlot, Callable[[Union[RenderPlotFunc, RenderPlotFuncAsync]], RenderPlot]
-]:
+) -> RenderPlot | Callable[[RenderPlotFunc | RenderPlotFuncAsync], RenderPlot]:
     """
     Reactively render a plot object as an HTML image.
 
@@ -367,7 +361,7 @@ def plot(
     ~shiny.render.image
     """
 
-    def wrapper(fn: Union[RenderPlotFunc, RenderPlotFuncAsync]) -> RenderPlot:
+    def wrapper(fn: RenderPlotFunc | RenderPlotFuncAsync) -> RenderPlot:
         if _utils.is_async_callable(fn):
             return RenderPlotAsync(fn, alt=alt, **kwargs)
         else:
@@ -382,11 +376,11 @@ def plot(
 # ======================================================================================
 # RenderImage
 # ======================================================================================
-RenderImageFunc = Callable[[], Union[ImgData, None]]
-RenderImageFuncAsync = Callable[[], Awaitable[Union[ImgData, None]]]
+RenderImageFunc = Callable[[], "ImgData | None"]
+RenderImageFuncAsync = Callable[[], Awaitable["ImgData | None"]]
 
 
-class RenderImage(RenderFunction[Union[ImgData, None], Union[ImgData, None]]):
+class RenderImage(RenderFunction["ImgData | None", "ImgData | None"]):
     def __init__(
         self,
         fn: RenderImageFunc,
@@ -400,11 +394,11 @@ class RenderImage(RenderFunction[Union[ImgData, None], Union[ImgData, None]]):
         # passed an async function, it will not change it.
         self._fn: RenderImageFuncAsync = _utils.wrap_async(fn)
 
-    def __call__(self) -> Union[ImgData, None]:
+    def __call__(self) -> ImgData | None:
         return _utils.run_coro_sync(self._run())
 
-    async def _run(self) -> Union[ImgData, None]:
-        res: Union[ImgData, None] = await self._fn()
+    async def _run(self) -> ImgData | None:
+        res: ImgData | None = await self._fn()
         if res is None:
             return None
 
@@ -422,19 +416,21 @@ class RenderImage(RenderFunction[Union[ImgData, None], Union[ImgData, None]]):
 
 
 class RenderImageAsync(
-    RenderImage, RenderFunctionAsync[Union[ImgData, None], Union[ImgData, None]]
+    RenderImage, RenderFunctionAsync["ImgData | None", "ImgData | None"]
 ):
     def __init__(self, fn: RenderImageFuncAsync, delete_file: bool = False) -> None:
         if not _utils.is_async_callable(fn):
             raise TypeError(self.__class__.__name__ + " requires an async function")
         super().__init__(typing.cast(RenderImageFunc, fn), delete_file=delete_file)
 
-    async def __call__(self) -> Union[ImgData, None]:  # type: ignore
+    async def __call__(  # pyright: ignore[reportIncompatibleMethodOverride]
+        self,
+    ) -> ImgData | None:
         return await self._run()
 
 
 @overload
-def image(fn: Union[RenderImageFunc, RenderImageFuncAsync]) -> RenderImage:
+def image(fn: RenderImageFunc | RenderImageFuncAsync) -> RenderImage:
     ...
 
 
@@ -442,17 +438,15 @@ def image(fn: Union[RenderImageFunc, RenderImageFuncAsync]) -> RenderImage:
 def image(
     *,
     delete_file: bool = False,
-) -> Callable[[Union[RenderImageFunc, RenderImageFuncAsync]], RenderImage]:
+) -> Callable[[RenderImageFunc | RenderImageFuncAsync], RenderImage]:
     ...
 
 
 def image(
-    fn: Optional[Union[RenderImageFunc, RenderImageFuncAsync]] = None,
+    fn: Optional[RenderImageFunc | RenderImageFuncAsync] = None,
     *,
     delete_file: bool = False,
-) -> Union[
-    RenderImage, Callable[[Union[RenderImageFunc, RenderImageFuncAsync]], RenderImage]
-]:
+) -> RenderImage | Callable[[RenderImageFunc | RenderImageFuncAsync], RenderImage]:
     """
     Reactively render a image file as an HTML image.
 
@@ -479,7 +473,7 @@ def image(
     ~shiny.render.plot
     """
 
-    def wrapper(fn: Union[RenderImageFunc, RenderImageFuncAsync]) -> RenderImage:
+    def wrapper(fn: RenderImageFunc | RenderImageFuncAsync) -> RenderImage:
         if _utils.is_async_callable(fn):
             return RenderImageAsync(fn, delete_file=delete_file)
         else:
@@ -510,7 +504,7 @@ class PandasCompatible(Protocol):
         ...
 
 
-class RenderTable(RenderFunction[object, Union["RenderedDeps", None]]):
+class RenderTable(RenderFunction[object, "RenderedDeps | None"]):
     def __init__(
         self,
         fn: RenderTableFunc,
@@ -530,10 +524,10 @@ class RenderTable(RenderFunction[object, Union["RenderedDeps", None]]):
         # passed an async function, it will not change it.
         self._fn: RenderTableFuncAsync = _utils.wrap_async(fn)
 
-    def __call__(self) -> Union["RenderedDeps", None]:
+    def __call__(self) -> RenderedDeps | None:
         return _utils.run_coro_sync(self._run())
 
-    async def _run(self) -> Union["RenderedDeps", None]:
+    async def _run(self) -> RenderedDeps | None:
         x = await self._fn()
 
         if x is None:
@@ -567,7 +561,7 @@ class RenderTable(RenderFunction[object, Union["RenderedDeps", None]]):
         return {"deps": [], "html": html}
 
 
-class RenderTableAsync(RenderTable, RenderFunctionAsync[object, Union[ImgData, None]]):
+class RenderTableAsync(RenderTable, RenderFunctionAsync[object, "ImgData | None"]):
     def __init__(
         self,
         fn: RenderTableFuncAsync,
@@ -587,12 +581,14 @@ class RenderTableAsync(RenderTable, RenderFunctionAsync[object, Union[ImgData, N
             **kwargs,
         )
 
-    async def __call__(self) -> Union["RenderedDeps", None]:  # type: ignore
+    async def __call__(  # pyright: ignore[reportIncompatibleMethodOverride]
+        self,
+    ) -> RenderedDeps | None:
         return await self._run()
 
 
 @overload
-def table(fn: Union[RenderTableFunc, RenderTableFuncAsync]) -> RenderTable:
+def table(fn: RenderTableFunc | RenderTableFuncAsync) -> RenderTable:
     ...
 
 
@@ -603,21 +599,19 @@ def table(
     classes: str = "table shiny-table w-auto",
     border: int = 0,
     **kwargs: Any,
-) -> Callable[[Union[RenderTableFunc, RenderTableFuncAsync]], RenderTable]:
+) -> Callable[[RenderTableFunc | RenderTableFuncAsync], RenderTable]:
     ...
 
 
 # TODO: Use more specific types for render.table
 def table(
-    fn: Optional[Union[RenderTableFunc, RenderTableFuncAsync]] = None,
+    fn: Optional[RenderTableFunc | RenderTableFuncAsync] = None,
     *,
     index: bool = False,
     classes: str = "table shiny-table w-auto",
     border: int = 0,
     **kwargs: Any,
-) -> Union[
-    RenderTable, Callable[[Union[RenderTableFunc, RenderTableFuncAsync]], RenderTable]
-]:
+) -> RenderTable | Callable[[RenderTableFunc | RenderTableFuncAsync], RenderTable]:
     """
     Reactively render a Pandas data frame object (or similar) as a basic HTML table.
 
@@ -657,7 +651,7 @@ def table(
     ~shiny.ui.output_table
     """
 
-    def wrapper(fn: Union[RenderTableFunc, RenderTableFuncAsync]) -> RenderTable:
+    def wrapper(fn: RenderTableFunc | RenderTableFuncAsync) -> RenderTable:
         if _utils.is_async_callable(fn):
             return RenderTableAsync(
                 fn, index=index, classes=classes, border=border, **kwargs
@@ -676,11 +670,11 @@ def table(
 # ======================================================================================
 # RenderUI
 # ======================================================================================
-RenderUIFunc = Callable[[], TagChildArg]
-RenderUIFuncAsync = Callable[[], Awaitable[TagChildArg]]
+RenderUIFunc = Callable[[], TagChild]
+RenderUIFuncAsync = Callable[[], Awaitable[TagChild]]
 
 
-class RenderUI(RenderFunction[TagChildArg, Union["RenderedDeps", None]]):
+class RenderUI(RenderFunction[TagChild, "RenderedDeps | None"]):
     def __init__(self, fn: RenderUIFunc) -> None:
         super().__init__(fn)
         # The Render*Async subclass will pass in an async function, but it tells the
@@ -688,48 +682,48 @@ class RenderUI(RenderFunction[TagChildArg, Union["RenderedDeps", None]]):
         # passed an async function, it will not change it.
         self._fn: RenderUIFuncAsync = _utils.wrap_async(fn)
 
-    def __call__(self) -> Union["RenderedDeps", None]:
+    def __call__(self) -> RenderedDeps | None:
         return _utils.run_coro_sync(self._run())
 
-    async def _run(self) -> Union["RenderedDeps", None]:
-        ui: TagChildArg = await self._fn()
+    async def _run(self) -> RenderedDeps | None:
+        ui: TagChild = await self._fn()
         if ui is None:
             return None
 
         return self._session._process_ui(ui)
 
 
-class RenderUIAsync(
-    RenderUI, RenderFunctionAsync[TagChildArg, Union["RenderedDeps", None]]
-):
+class RenderUIAsync(RenderUI, RenderFunctionAsync[TagChild, "RenderedDeps| None"]):
     def __init__(self, fn: RenderUIFuncAsync) -> None:
         if not _utils.is_async_callable(fn):
             raise TypeError(self.__class__.__name__ + " requires an async function")
         super().__init__(typing.cast(RenderUIFunc, fn))
 
-    async def __call__(self) -> Union["RenderedDeps", None]:  # type: ignore
+    async def __call__(  # pyright: ignore[reportIncompatibleMethodOverride]
+        self,
+    ) -> RenderedDeps | None:
         return await self._run()
 
 
 @overload
-def ui(fn: Union[RenderUIFunc, RenderUIFuncAsync]) -> RenderUI:
+def ui(fn: RenderUIFunc | RenderUIFuncAsync) -> RenderUI:
     ...
 
 
 @overload
-def ui() -> Callable[[Union[RenderUIFunc, RenderUIFuncAsync]], RenderUI]:
+def ui() -> Callable[[RenderUIFunc | RenderUIFuncAsync], RenderUI]:
     ...
 
 
 def ui(
-    fn: Optional[Union[RenderUIFunc, RenderUIFuncAsync]] = None
-) -> Union[RenderUI, Callable[[Union[RenderUIFunc, RenderUIFuncAsync]], RenderUI]]:
+    fn: Optional[RenderUIFunc | RenderUIFuncAsync] = None,
+) -> RenderUI | Callable[[RenderUIFunc | RenderUIFuncAsync], RenderUI]:
     """
     Reactively render HTML content.
 
     Returns
     -------
-    A decorator for a function that returns an object of type `~shiny.ui.TagChildArg`.
+    A decorator for a function that returns an object of type `~shiny.ui.TagChild`.
 
     Tip
     ----
@@ -744,7 +738,7 @@ def ui(
     """
 
     def wrapper(
-        fn: Union[Callable[[], TagChildArg], Callable[[], Awaitable[TagChildArg]]]
+        fn: Callable[[], TagChild] | Callable[[], Awaitable[TagChild]]
     ) -> RenderUI:
         if _utils.is_async_callable(fn):
             return RenderUIAsync(fn)
