@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 __all__ = (
     "update_action_button",
     "update_action_link",
@@ -20,7 +22,7 @@ import json
 import re
 import sys
 from datetime import date
-from typing import List, Mapping, Optional, Tuple, Union
+from typing import Mapping, Optional
 
 from starlette.requests import Request
 from starlette.responses import JSONResponse, Response
@@ -35,7 +37,7 @@ if sys.version_info >= (3, 8):
 else:
     from typing_extensions import Literal, TypedDict
 
-from htmltools import TagChildArg
+from htmltools import TagChild
 
 from .._docstring import add_example, doc_format
 from .._namespaces import resolve_id
@@ -74,7 +76,7 @@ def update_action_button(
     id: str,
     *,
     label: Optional[str] = None,
-    icon: TagChildArg = None,
+    icon: TagChild = None,
     session: Optional[Session] = None,
 ) -> None:
     """
@@ -102,7 +104,7 @@ def update_action_button(
     """
 
     session = require_active_session(session)
-    # TODO: supporting a TagChildArg for label would require changes to shiny.js
+    # TODO: supporting a TagChild for label would require changes to shiny.js
     # https://github.com/rstudio/shiny/issues/1140
     msg = {"label": label, "icon": session._process_ui(icon)["html"] if icon else None}
     session.send_input_message(id, drop_none(msg))
@@ -111,6 +113,7 @@ def update_action_button(
 update_action_link = update_action_button
 update_action_link.__doc__ = update_action_button.__doc__
 
+
 # -----------------------------------------------------------------------------
 # input_check_radio.py
 # -----------------------------------------------------------------------------
@@ -196,7 +199,7 @@ def update_checkbox_group(
     *,
     label: Optional[str] = None,
     choices: Optional[ChoicesArg] = None,
-    selected: Optional[Union[str, List[str]]] = None,
+    selected: Optional[str | list[str]] = None,
     inline: bool = False,
     session: Optional[Session] = None,
 ) -> None:
@@ -299,7 +302,7 @@ def _update_choice_input(
     type: Literal["checkbox", "radio"],
     label: Optional[str] = None,
     choices: Optional[ChoicesArg] = None,
-    selected: Optional[Union[str, List[str]]] = None,
+    selected: Optional[str | list[str]] = None,
     inline: bool = False,
     session: Optional[Session] = None,
 ) -> None:
@@ -327,9 +330,9 @@ def update_date(
     id: str,
     *,
     label: Optional[str] = None,
-    value: Optional[Union[date, str]] = None,
-    min: Optional[Union[date, str]] = None,
-    max: Optional[Union[date, str]] = None,
+    value: Optional[date | str] = None,
+    min: Optional[date | str] = None,
+    max: Optional[date | str] = None,
     session: Optional[Session] = None,
 ) -> None:
     """
@@ -377,10 +380,10 @@ def update_date_range(
     id: str,
     *,
     label: Optional[str] = None,
-    start: Optional[Union[date, str]] = None,
-    end: Optional[Union[date, str]] = None,
-    min: Optional[Union[date, str]] = None,
-    max: Optional[Union[date, str]] = None,
+    start: Optional[date | str] = None,
+    end: Optional[date | str] = None,
+    min: Optional[date | str] = None,
+    max: Optional[date | str] = None,
     session: Optional[Session] = None,
 ) -> None:
     """
@@ -493,7 +496,7 @@ def update_select(
     *,
     label: Optional[str] = None,
     choices: Optional[SelectChoicesArg] = None,
-    selected: Optional[Union[str, List[str]]] = None,
+    selected: Optional[str | list[str]] = None,
     session: Optional[Session] = None,
 ) -> None:
     """
@@ -562,7 +565,7 @@ def update_selectize(
     *,
     label: Optional[str] = None,
     choices: Optional[SelectChoicesArg] = None,
-    selected: Optional[Union[str, List[str]]] = None,
+    selected: Optional[str | list[str]] = None,
     # TODO: we need the equivalent of base::I()/htmlwidgets::JS() for marking strings as strings to be evaluated
     # options: Optional[Dict[str, str]] = None,
     server: bool = False,
@@ -611,9 +614,9 @@ def update_selectize(
 
     # Transform choices to a list of dicts (this is the form the client wants)
     # [{"label": "Foo", "value": "foo", "optgroup": "foo"}, ...]
-    flat_choices: List[FlatSelectChoice] = []
+    flat_choices: list[FlatSelectChoice] = []
     if choices is not None:
-        for (k, v) in _normalize_choices(choices).items():
+        for k, v in _normalize_choices(choices).items():
             if not isinstance(v, Mapping):
                 flat_choices.append(
                     FlatSelectChoice(value=k, label=session._process_ui(v)["html"])
@@ -661,7 +664,7 @@ def update_selectize(
         conjunction = any if qparams.get("conju", "and") == "or" else all
 
         # i.e. searchFields (defaults to ['label'])
-        search_fields: List[str] = json.loads(qparams.get("field", "['label']"))
+        search_fields: list[str] = json.loads(qparams.get("field", "['label']"))
         if len(search_fields) == 0:
             raise ValueError("The selectize.js searchFields option must be non-empty")
 
@@ -683,7 +686,7 @@ def update_selectize(
                 "The selectize.js valueField option must be set to 'value'"
             )
 
-        filtered_choices: List[FlatSelectChoice] = []
+        filtered_choices: list[FlatSelectChoice] = []
         for choice in flat_choices:
             # Short-circuit if we've reached the max number of options
             if (len(filtered_choices) + len(selected_choices)) > max_options:
@@ -728,9 +731,7 @@ def update_slider(
     id: str,
     *,
     label: Optional[str] = None,
-    value: Optional[
-        Union[SliderValueArg, Tuple[SliderValueArg, SliderValueArg]]
-    ] = None,
+    value: Optional[SliderValueArg | tuple[SliderValueArg, SliderValueArg]] = None,
     min: Optional[SliderValueArg] = None,
     max: Optional[SliderValueArg] = None,
     step: Optional[SliderStepArg] = None,
@@ -861,6 +862,7 @@ update_text_area.__doc__ = update_text.__doc__
 # navs.py
 # -----------------------------------------------------------------------------
 
+
 # TODO: we should probably provide a nav_select() alias for this as well
 @add_example()
 @doc_format(note=_note)
