@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 __all__ = (
     "nav",
     "nav_menu",
@@ -15,21 +17,22 @@ __all__ = (
 import copy
 import re
 import sys
-from typing import List, Optional, Tuple, Union, cast, Dict, Any
+from typing import Any, Optional, cast
 
 if sys.version_info >= (3, 8):
     from typing import Literal
 else:
     from typing_extensions import Literal
 
-from htmltools import tags, Tag, TagList, TagChildArg, div
+from htmltools import Tag, TagChild, TagList, div, tags
 
-from ._bootstrap import row, column
 from .._docstring import add_example
-from ._html_dependencies import bootstrap_deps
 from .._namespaces import resolve_id
-from ..types import NavSetArg
 from .._utils import private_random_int
+from ..types import NavSetArg
+from ._bootstrap import column, row
+from ._html_dependencies import bootstrap_deps
+
 
 # -----------------------------------------------------------------------------
 # Navigation items
@@ -44,9 +47,8 @@ class Nav:
         self.content = content
 
     def resolve(
-        self, selected: Optional[str], context: Dict[str, Any]
-    ) -> Tuple[TagChildArg, TagChildArg]:
-
+        self, selected: Optional[str], context: dict[str, Any]
+    ) -> tuple[TagChild, TagChild]:
         # Nothing to do for nav_control()/nav_spacer()
         if self.content is None:
             return self.nav, None
@@ -92,10 +94,10 @@ class Nav:
 
 @add_example()
 def nav(
-    title: TagChildArg,
-    *args: TagChildArg,
+    title: TagChild,
+    *args: TagChild,
     value: Optional[str] = None,
-    icon: TagChildArg = None,
+    icon: TagChild = None,
 ) -> Nav:
     """
     Create a nav item pointing to some internal content.
@@ -147,7 +149,7 @@ def nav(
     )
 
 
-def nav_control(*args: TagChildArg) -> Nav:
+def nav_control(*args: TagChild) -> Nav:
     """
     Place a control in the navigation container.
 
@@ -199,15 +201,15 @@ def nav_spacer() -> Nav:
 
 
 class NavMenu:
-    nav_controls: List[NavSetArg]
-    title: TagChildArg
+    nav_controls: list[NavSetArg]
+    title: TagChild
     value: str
     align: Literal["left", "right"]
 
     def __init__(
         self,
-        *args: Union[NavSetArg, str],
-        title: TagChildArg,
+        *args: NavSetArg | str,
+        title: TagChild,
         value: str,
         align: Literal["left", "right"] = "left",
     ) -> None:
@@ -219,8 +221,8 @@ class NavMenu:
     def resolve(
         self,
         selected: Optional[str],
-        context: Dict[str, Any],
-    ) -> Tuple[TagChildArg, TagChildArg]:
+        context: dict[str, Any],
+    ) -> tuple[TagChild, TagChild]:
         nav, content = render_navset(
             *self.nav_controls,
             ul_class=f"dropdown-menu {'dropdown-menu-right' if self.align == 'right' else ''}",
@@ -264,7 +266,7 @@ class NavMenu:
         raise NotImplementedError("nav_menu() must appear within navset_*() container.")
 
 
-def menu_string_as_nav(x: Union[str, NavSetArg]) -> NavSetArg:
+def menu_string_as_nav(x: str | NavSetArg) -> NavSetArg:
     if not isinstance(x, str):
         return x
 
@@ -277,10 +279,10 @@ def menu_string_as_nav(x: Union[str, NavSetArg]) -> NavSetArg:
 
 
 def nav_menu(
-    title: TagChildArg,
-    *args: Union[Nav, str],
+    title: TagChild,
+    *args: Nav | str,
     value: Optional[str] = None,
-    icon: TagChildArg = None,
+    icon: TagChild = None,
     align: Literal["left", "right"] = "left",
 ) -> NavMenu:
     """
@@ -337,12 +339,12 @@ def nav_menu(
 
 
 class NavSet:
-    args: Tuple[NavSetArg]
+    args: tuple[NavSetArg]
     ul_class: str
     id: Optional[str]
     selected: Optional[str]
-    header: TagChildArg
-    footer: TagChildArg
+    header: TagChild
+    footer: TagChild
 
     def __init__(
         self,
@@ -350,8 +352,8 @@ class NavSet:
         ul_class: str,
         id: Optional[str],
         selected: Optional[str],
-        header: TagChildArg = None,
-        footer: TagChildArg = None,
+        header: TagChild = None,
+        footer: TagChild = None,
     ) -> None:
         self.args = args
         self.ul_class = ul_class
@@ -360,7 +362,7 @@ class NavSet:
         self.header = header
         self.footer = footer
 
-    def tagify(self) -> Union[TagList, Tag]:
+    def tagify(self) -> TagList | Tag:
         id = self.id
         ul_class = self.ul_class
         if id is not None:
@@ -371,7 +373,7 @@ class NavSet:
         )
         return self.layout(nav, content)
 
-    def layout(self, nav: TagChildArg, content: TagChildArg) -> Union[TagList, Tag]:
+    def layout(self, nav: TagChild, content: TagChild) -> TagList | Tag:
         return TagList(nav, self.header, content, self.footer)
 
 
@@ -382,8 +384,8 @@ def navset_tab(
     *args: NavSetArg,
     id: Optional[str] = None,
     selected: Optional[str] = None,
-    header: TagChildArg = None,
-    footer: TagChildArg = None,
+    header: TagChild = None,
+    footer: TagChild = None,
 ) -> NavSet:
     """
     Render nav items as a tabset.
@@ -435,8 +437,8 @@ def navset_pill(
     *args: NavSetArg,
     id: Optional[str] = None,
     selected: Optional[str] = None,
-    header: TagChildArg = None,
-    footer: TagChildArg = None,
+    header: TagChild = None,
+    footer: TagChild = None,
 ) -> NavSet:
     """
     Render nav items as a pillset.
@@ -488,8 +490,8 @@ def navset_hidden(
     *args: NavSetArg,
     id: Optional[str] = None,
     selected: Optional[str] = None,
-    header: TagChildArg = None,
-    footer: TagChildArg = None,
+    header: TagChild = None,
+    footer: TagChild = None,
 ) -> NavSet:
     """
     Render nav contents without the nav items.
@@ -542,8 +544,8 @@ class NavSetCard(NavSet):
         ul_class: str,
         id: Optional[str],
         selected: Optional[str],
-        header: TagChildArg = None,
-        footer: TagChildArg = None,
+        header: TagChild = None,
+        footer: TagChild = None,
         placement: Literal["above", "below"] = "above",
     ) -> None:
         super().__init__(
@@ -556,7 +558,7 @@ class NavSetCard(NavSet):
         )
         self.placement = placement
 
-    def layout(self, nav: TagChildArg, content: TagChildArg) -> Tag:
+    def layout(self, nav: TagChild, content: TagChild) -> Tag:
         if self.placement == "below":
             return card(self.header, content, self.footer, footer=nav)
         else:
@@ -567,8 +569,8 @@ def navset_tab_card(
     *args: NavSetArg,
     id: Optional[str] = None,
     selected: Optional[str] = None,
-    header: TagChildArg = None,
-    footer: TagChildArg = None,
+    header: TagChild = None,
+    footer: TagChild = None,
 ) -> NavSetCard:
     """
     Render nav items as a tabset inside a card container.
@@ -620,8 +622,8 @@ def navset_pill_card(
     *args: NavSetArg,
     id: Optional[str] = None,
     selected: Optional[str] = None,
-    header: TagChildArg = None,
-    footer: TagChildArg = None,
+    header: TagChild = None,
+    footer: TagChild = None,
     placement: Literal["above", "below"] = "above",
 ) -> NavSetCard:
     """
@@ -674,7 +676,7 @@ def navset_pill_card(
 
 class NavSetPillList(NavSet):
     well: bool
-    widths: Tuple[int, int]
+    widths: tuple[int, int]
 
     def __init__(
         self,
@@ -682,10 +684,10 @@ class NavSetPillList(NavSet):
         ul_class: str,
         id: Optional[str],
         selected: Optional[str],
-        header: TagChildArg = None,
-        footer: TagChildArg = None,
+        header: TagChild = None,
+        footer: TagChild = None,
         well: bool = True,
-        widths: Tuple[int, int] = (4, 8),
+        widths: tuple[int, int] = (4, 8),
     ) -> None:
         super().__init__(
             *args,
@@ -698,7 +700,7 @@ class NavSetPillList(NavSet):
         self.well = well
         self.widths = widths
 
-    def layout(self, nav: TagChildArg, content: TagChildArg) -> Tag:
+    def layout(self, nav: TagChild, content: TagChild) -> Tag:
         widths = self.widths
         return row(
             column(widths[0], nav, class_="well" if self.well else None),
@@ -710,10 +712,10 @@ def navset_pill_list(
     *args: NavSetArg,
     id: Optional[str] = None,
     selected: Optional[str] = None,
-    header: TagChildArg = None,
-    footer: TagChildArg = None,
+    header: TagChild = None,
+    footer: TagChild = None,
     well: bool = True,
-    widths: Tuple[int, int] = (4, 8),
+    widths: tuple[int, int] = (4, 8),
 ) -> NavSet:
     """
     Render nav items as a vertical pillset.
@@ -771,7 +773,7 @@ def navset_pill_list(
 
 
 class NavSetBar(NavSet):
-    title: TagChildArg
+    title: TagChild
     position: Literal["static-top", "fixed-top", "fixed-bottom", "sticky-top"]
     bg: Optional[str]
     inverse: bool
@@ -782,14 +784,14 @@ class NavSetBar(NavSet):
         self,
         *args: NavSetArg,
         ul_class: str,
-        title: TagChildArg,
+        title: TagChild,
         id: Optional[str],
         selected: Optional[str],
         position: Literal[
             "static-top", "fixed-top", "fixed-bottom", "sticky-top"
         ] = "static-top",
-        header: TagChildArg = None,
-        footer: TagChildArg = None,
+        header: TagChild = None,
+        footer: TagChild = None,
         bg: Optional[str] = None,
         # TODO: default to 'auto', like we have in R (parse color via webcolors?)
         inverse: bool = False,
@@ -811,7 +813,7 @@ class NavSetBar(NavSet):
         self.collapsible = collapsible
         self.fluid = fluid
 
-    def layout(self, nav: TagChildArg, content: TagChildArg) -> TagList:
+    def layout(self, nav: TagChild, content: TagChild) -> TagList:
         nav_container = div(
             {"class": "container-fluid" if self.fluid else "container"},
             tags.a({"class": "navbar-brand", "href": "#"}, self.title),
@@ -858,14 +860,14 @@ class NavSetBar(NavSet):
 
 def navset_bar(
     *args: NavSetArg,
-    title: TagChildArg,
+    title: TagChild,
     id: Optional[str] = None,
     selected: Optional[str] = None,
     position: Literal[
         "static-top", "fixed-top", "fixed-bottom", "sticky-top"
     ] = "static-top",
-    header: TagChildArg = None,
-    footer: TagChildArg = None,
+    header: TagChild = None,
+    footer: TagChild = None,
     bg: Optional[str] = None,
     # TODO: default to 'auto', like we have in R (parse color via webcolors?)
     inverse: bool = False,
@@ -950,9 +952,8 @@ def render_navset(
     ul_class: str,
     id: Optional[str],
     selected: Optional[str],
-    context: Dict[str, Any],
-) -> Tuple[Tag, Tag]:
-
+    context: dict[str, Any],
+) -> tuple[Tag, Tag]:
     tabsetid = private_random_int(1000, 10000)
 
     # If the user hasn't provided a selected value, use the first one
@@ -974,9 +975,7 @@ def render_navset(
     return ul_tag, div_tag
 
 
-def card(
-    *args: TagChildArg, header: TagChildArg = None, footer: TagChildArg = None
-) -> Tag:
+def card(*args: TagChild, header: TagChild = None, footer: TagChild = None) -> Tag:
     if header:
         header = div(header, class_="card-header")
     if footer:
